package elvisDoc;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.Enumeration;
import java.util.Hashtable;

/**
 * Format of the serialized document is very simple.
 * The first object is an integer of the number of elements, followed
 * by the elements themselves.
 * 
 * @author ritza
 *
 */
public class ElvisDocSerializeDoc {
	
	private final static String serializeDir = File.separator+"SerializeDoc"+File.separator;
	
	public ElvisDocSerializeDoc(Hashtable<String, elvisDocItem> docList, 
			String outputPath) throws IOException {
		outputPath += serializeDir;
		int numElements = countElements(docList, 0);
		ObjectOutputStream stream = new ObjectOutputStream(new FileOutputStream(outputPath));
		stream.writeInt(numElements);
		serializeRecurse(docList,"root",stream);
		stream.close();
	}
	
	private void serializeRecurse(Hashtable<String, elvisDocItem> docList, String key,
			ObjectOutputStream stream) throws IOException{
		elvisDocItem eitem = docList.get(key);
		stream.writeObject(eitem);
		Enumeration<elvisDocItem> echildren = eitem.children.elements();
		while (echildren.hasMoreElements()) {
			elvisDocItem eid = (elvisDocItem)echildren.nextElement();
			serializeRecurse(eitem.children,eid.name,stream);
		}
	}
	
	private int countElements(Hashtable<String, elvisDocItem> docList, int count) {
		Enumeration<elvisDocItem> e = docList.elements();
		while (e.hasMoreElements()) {
			elvisDocItem edi = (elvisDocItem)e.nextElement();
			count++;
			countElements(edi.children,count); 
		}
		return count;
	}
	
	public static File getSerializeDocNameFromUserDir() throws FileNotFoundException {
		// assumes using the user directory.
	
		String pathname = System.getProperty("user.dir")+serializeDir;
		File file = new File(pathname);
		if (!file.exists())
			throw new FileNotFoundException("Serialize Document hasn't been created in the\n" +
				"user directory yet.");
		return file;
	}
}
