package errors;

import types.Widget;

/** An unrecognized attribute name.
 *  Has no variables beyond those of the superclass, but formats the message
 *  according to the widget and the attribute name. */
public class BadAttributeNameException extends BadAttributeException {
    /** @param widget The widget with the invalid attribute name. 
     *  @param attrName The invalid attribute name. */
    public BadAttributeNameException(Widget widget, String attrName) {
        super(formatMsg(widget, attrName), widget, attrName);
    }
    
    private static String formatMsg(Widget widget, String attrName) {
        return String.format(
                "%s: unrecognized attribute name for widget type %s", attrName,
                widget.type);
    }
}
