package errors;

import types.Widget;

/**
 * BadChildWidgetException is thrown when there is an illegal child.  
 * Thrown by WidgetDefs.
 */
public class BadChildWidgetException extends GUIFileException {

    /** The parent widget for which the child widget is invalid. */
    public final Widget widget;
    /** The child widget in violation. */
    public final String childType;
    
	/**
     * Report an erroneous child widget for a given widget.
	 * @param widget The parent widget.
     * @param child The child widget in violation.
	 */
	public BadChildWidgetException(Widget widget, Widget child) {
		super(formatMsg(widget.type, child.type), widget.lineNum);
		this.widget = widget;
        this.childType = child.type;
	}
    
    /**
     * Report an erroneus widget type for the child of a given widget.
     * @param widget The parent widget.
     * @param childType The type of the child widget in violation.
     */
    public BadChildWidgetException(Widget widget, String childType) {
        super(formatMsg(widget.type, childType), widget.lineNum);
        this.widget = widget;
        this.childType = childType;
    }
    
    /**
     * Report that a given widget cannot have <i>any</i> children.
     * @param widget The parent widget.
     */
    public BadChildWidgetException(Widget widget) {
        super(formatMsg(widget.type, null), widget.lineNum);
        this.widget = widget;
        this.childType = "<unknown>";
    }
    
    private static String formatMsg(String widgetType, String childType) {
        if (childType == null)
            return String.format(
                    "widget of type %s cannot have a child widget",
                    widgetType);
        return String.format(
                "widget of type %s cannot have a child of type %s",
                widgetType, childType);
    }

}
