package errors;

/** An invalid attribute value.
 *  This will likely be thrown only by library code. As such, it permits an
 *  extra error message to be passed along, which is expected to describe the
 *  problem - for example, it might be "width must be an integer". */
public class BadOptionValueException extends BadOptionException {
    /** The value of the option. */
    public final String value;

    /** @param lineNum The widget with the invalid attribute.
     *  @param name The name of the invalid attribute.
     *  @param value The invalid value in question. */
    public BadOptionValueException(int lineNum, String name,
            String value) {
        super(formatMsg(name, value), lineNum, name);
        this.value = value;
    }

    /** @param msg The details to be appended to the error message.
     *  @param lineNum The widget with the invalid attribute.
     *  @param name The name of the invalid attribute.
     *  @param value The invalid value in question. */
    public BadOptionValueException(String msg, int lineNum,
            String name, String value) {
        super(formatMsg(msg, name, value), lineNum, name);
        this.value = value;
    }

    private static String formatMsg(String attrName, String attrValue) {
        return String.format(
                "%s: invalid value for option %s", attrValue, attrName);
    }

    private static String formatMsg(String msg, String name, String value) {
        return formatMsg(name, value) + ": " + msg;
    }
}
