package errors;

import types.Widget;

/**
 * Checked exception thrown when a .gui file declares a widget of an unknown
 * type or an abstract widget type (which is not a user-visible feature). 
 */
public class BadWidgetTypeException extends GUIFileException {
	/**
	 * @param widget The widget in violation.
	 */
	public BadWidgetTypeException(Widget widget) {
		super(formatMsg(widget), widget.lineNum);
	}
    
    /**
     * @param msg An explanatory error message.
     * @param widget The widget in violation.
     */
    public BadWidgetTypeException(String msg, Widget widget) {
        super(formatMsg(widget, msg), widget.lineNum);
    }

    private static String formatMsg(Widget widget, String msg) {
        return String.format("%s: %s", widget.type, msg);
    }
    private static String formatMsg(Widget widget) {
        return formatMsg(widget, "widget type not recognized");
    }
}
