package errors;

import types.Widget;

/** A second declaration for the same identifier for different widgets. */
public class DuplicateIdentifierException extends GUIFileException {
    /** The widget that was first declared with the identifier. */
    public final Widget original;
    /** The new widget, declared with the same identifier. */
    public final Widget dupe;

    /** @param original The original widget. 
     *  @param dupe The new widget. */
    public DuplicateIdentifierException(Widget original, Widget dupe) {
        super(formatMsg(original, dupe), dupe.lineNum);
        this.original = original;
        this.dupe = dupe;
    }

    private static String formatMsg(Widget original, Widget dupe) {
        return String.format("%s: identifier already used at line %d",
                dupe.id, original.lineNum);
    }
}
