package errors;

import parser.ParseException;
import parser.Token;

/** An error caught by the parser. All such exceptions originate as
 *  <code>parser.gen.ParseException</code> or
 *  <code>parser.gen.TokenMgrError</code> instances. Therefore this class's
 *  principal constructors take one and adapt it to this public exception
 *  hierarchy. */
public class SyntaxException extends GUIFileException {
    /** The original exception. */
    
    public final Throwable original;
    
    private static int firstLineOrNegativeOne(Token token) {
        if (token == null)
            return -1;
        if (token.next == null)
            return -1;
        return token.next.beginLine;
    }
    
    /** @param original The <code>parser.gen.ParseException</code> that
     * originally reported the error. */
    public SyntaxException(ParseException original) {
        super(original.getMessage(),
                firstLineOrNegativeOne(original.currentToken));
        this.original = original;
    }

    /** Override the message provided by the original exception.
     *  @param msg The error message. It need not contain the line number; this
     *      will be extracted from the original exception.
     *  @param original The original exception. */
    public SyntaxException(String msg, ParseException original) {
        super(msg, firstLineOrNegativeOne(original.currentToken));
        this.original = original;
    }

}
