package learningTool;

import interpreter.Assembler;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.io.*;
import java.util.*;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeNode;

import learningTool.LearningToolGUI.FileChooserAnswer;

import elvisDoc.ElvisDocSerializeDoc;
import elvisDoc.elvisDocAttribute;
import elvisDoc.elvisDocItem;
import errors.GUIFileException;
import types.ErrorHandler;
import util.Shell;

public class LearningTool implements Controller {
	
	private final LearningToolGUI lt;
	
	private boolean suppressModifiers;
	private boolean executable;
	//private String fileName = null;
	private String outputFileName = null;
	private final Hashtable<String, elvisDocItem> docList;
	
	// TODO: Eliminate hardcoding when ComboBox for languages is implemented.
	private String destinationLanguage = "Java";

	public static void main(String[] args) throws FileNotFoundException, IOException, ClassNotFoundException {
		new LearningTool();
	}
	
	/**
	 * Constructor:
	 * 
	 * Tree nodes are populated here; use the SerializeDoc serialized file
	 * to extract the nodes and reconstruct the hashtable.
	 * @throws IOException, FileNotFoundException 
	 * @throws ClassNotFoundException 
	 */
	public LearningTool() throws IOException, FileNotFoundException, ClassNotFoundException {
		lt = new LearningToolGUI(this);
		docList = new Hashtable<String, elvisDocItem>();
		File file = ElvisDocSerializeDoc.getSerializeDocNameFromUserDir();
		ObjectInputStream stream = new ObjectInputStream(
				new FileInputStream(file));
		int numElements = stream.readInt();
		
		DefaultMutableTreeNode root = lt.AUX$tree$root;
		root.setUserObject(((elvisDocItem)stream.readObject()).name);
		DefaultMutableTreeNode currentNode = root;
		DefaultMutableTreeNode tempNode;
		for (int i=1; i<numElements;i++) {
			elvisDocItem item = (elvisDocItem)stream.readObject();
			String currentParent = item.parentName;
			if (!(currentParent.equals(currentNode.getUserObject()))) {
				tempNode = currentNode = getCorrectNode(
						(DefaultMutableTreeNode)currentNode.getFirstChild(), 
						root,currentParent);
				if (tempNode == null)
					tempNode = getCorrectNode(
							(DefaultMutableTreeNode)currentNode.getParent(), 
							root,currentParent);
				if (tempNode == null)
					System.err.println(item.parentName +" is not in tree!");
				else
					currentNode = tempNode;
			}
			if (currentParent.equals(currentNode.getUserObject())) {
				currentNode.add(new DefaultMutableTreeNode(item.name));
				docList.put(item.name,item);
			}		
		}
		root.setUserObject("Elvis Widget API Documentation");
		lt.tree.expandRow(0);
		lt.tree.validate();
	}
	
	/**
	 * Returns the correct Default Mutable Tree Node or null if it doesn't exist.
	 * @param sibling
	 */
	private DefaultMutableTreeNode getCorrectNode(DefaultMutableTreeNode sibling,
			DefaultMutableTreeNode root, String key)
	{
		DefaultMutableTreeNode currentNode = sibling;
		// get to first node in siblings.
		while (currentNode.getPreviousSibling() != null) {
			currentNode = currentNode.getPreviousSibling();
		}
		
		// boolean to test if node is root node.
		boolean isRoot = false;
		if (currentNode.equals(root))
			isRoot = true;
		
		// iterate through siblings to see if node is there.
		while (currentNode != null) {
			if (currentNode.getUserObject().equals(key))
				return currentNode;
			currentNode = currentNode.getNextSibling();
		}

		if (isRoot)	return null;
		
		else { // recurse.
			return getCorrectNode(
					(DefaultMutableTreeNode)sibling.getParent(), 
					root, key);
		}
	}
	
	public void newDoc(Component widget, ActionEvent event) {
		lt.elvisText.setText("");
		lt.elvisDocName.setText("");
		lt.javaText.setText("");
		lt.javaDocName.setText("");
	}

	public void open(Component widget, ActionEvent event) {
		FileChooserAnswer answer = lt.runFileChooserAsOpen(lt.fileChooser, lt.window);
		if (answer.equals(FileChooserAnswer.OK)) {
			 String file = lt.fileChooser.getSelectedFile().getPath();
			 try {
				Scanner scanner = new Scanner(new File(file));
				lt.elvisText.setText("");
				lt.javaText.setText("");
				while(scanner.hasNextLine()) {
					lt.elvisText.append(scanner.nextLine());
					lt.elvisText.append("\n");
				}
				lt.elvisDocName.setText(file);
				lt.javaDocName.setText("");
			} catch (FileNotFoundException e) {
				System.err.println("File " + file + "not found.");
				e.printStackTrace();
			}
		}
		else if (answer.equals(FileChooserAnswer.ERROR)) 
			System.err.println("Error running FileChooser.");
	}

	public void save(Component widget, ActionEvent event) {
		if(lt.elvisDocName.getText().equals("")) {
			saveAs(null,null);
			return;
		}	
		writeElvisFile(new File(lt.elvisDocName.getText()));
	}

	public void saveAs(Component widget, ActionEvent event) {
		FileChooserAnswer answer = lt.runFileChooserAsSave(lt.fileChooser, lt.window);
		if (answer.equals(FileChooserAnswer.OK)) {
			File file = lt.fileChooser.getSelectedFile();
			writeElvisFile(file);
			lt.elvisDocName.setText(file.getPath());    
		}
		else if (answer.equals(FileChooserAnswer.ERROR)) 
			System.err.println("Error running FileChooser.");
	}
	
	
	/**
	 * Writes an Elvis file to the specified filename, taken either
	 * from the "Save As..." FileChooser or from the filename label at
	 * the bottom of the Elvis text area; depends on whether "save" or 
	 * "saveAs" calls this method.
	 * 
	 * @param file
	 */
	private void writeElvisFile(File file) {
		BufferedWriter writer = null;
		try {
			writer = new BufferedWriter(new FileWriter(file));
			writer.write(lt.elvisText.getText());
		} catch (IOException e) {
			System.out.println("Error writing to file.");
			e.printStackTrace();
		}
		
		if (writer != null) {
			try {
				writer.close();
			} catch (IOException e) {
				System.err.println("Error closing file.");
				e.printStackTrace();
			}
		}
	}

	/**
	 * Writes an Java file to the specified filename.
	 * 
	 * @param file
	 */
	private void writeJavaFile(File file) {
		BufferedWriter writer = null;
		try {
			writer = new BufferedWriter(new FileWriter(file));
			writer.write("package generatedJavaFiles;\n\n"+lt.javaText.getText());
		} catch (IOException e) {
			System.out.println("Error writing to file.");
			e.printStackTrace();
		}
		
		if (writer != null) {
			try {
				writer.close();
			} catch (IOException e) {
				System.err.println("Error closing file.");
				e.printStackTrace();
			}
		}
	}
		
	public void quit(Component widget, ActionEvent event) {
		lt.window.dispose();
	}

	/**
	 * For copy, cut, and paste functionality, we need to check 
	 * both editable text areas to see what is selected.
	 */
	public void copy(Component widget, ActionEvent event) {
		if (lt.elvisText.getSelectedText() != null)
			lt.elvisText.copy();
		else
			lt.javaText.copy();
	}

	public void cut(Component widget, ActionEvent event) {
		lt.elvisText.cut();
	}

	public void paste(Component widget, ActionEvent event) {
		lt.elvisText.paste();
	}

	public void selElvis(Component widget, ActionEvent event) {
		lt.elvisText.selectAll();
	}

	public void selJava(Component widget, ActionEvent event) {
		lt.javaText.selectAll();
	}

	public void help(Component widget, ActionEvent event) {
		lt.triggerhelpDialog();
	}

	public void tutorial(Component widget, ActionEvent event) {
		lt.triggertutorialDialog();
	}

	public void about(Component widget, ActionEvent event) {
		lt.triggeraboutDialog();
	}

	public void compile(Component widget, ActionEvent event) {
		save(null,null);

		lt.errorText.setText("");
		
		Assembler assembler = new Assembler();
		final String newJavaName;
		try {
			LTErrorHandler errorHandler = new LTErrorHandler(lt);
			outputFileName = assembler.generateCode(errorHandler, 
                    lt.elvisDocName.getText(), destinationLanguage,
						executable, suppressModifiers);
			if(!errorHandler.anyErrors())
			{
				newJavaName = System.getProperty("user.dir")+File.separator+"generatedJavaFiles"+File.separator+outputFileName;
				Scanner scanner = new Scanner(new File(outputFileName));
				lt.javaDocName.setText(newJavaName);
				lt.javaText.setText("");
				while(scanner.hasNextLine()) {
					lt.javaText.append(scanner.nextLine());
					lt.javaText.append("\n");
				}
			} else {
				lt.errorText.append("\n"+errorHandler.totalErrors()+" errors found.");
                return;
			}
			
		} catch (Throwable e) {
			lt.errorText.append(e.getMessage());
            return;
		}
		writeJavaFile(new File(newJavaName));
	}

	public void suppressModifiers(Component widget, ActionEvent event) {
		if (((JCheckBox) widget).getModel().isSelected())
			suppressModifiers = true;
		else
			suppressModifiers = false;
	}

	public void executable(Component widget, ActionEvent event) {
		if (((JCheckBox) widget).getModel().isSelected())
			executable = true;
		else
			executable = false;
	
	}

	public void language(Component widget, ActionEvent event) {
		destinationLanguage = (String)((JComboBox) widget).getModel().getSelectedItem();
		lt.errorText.append("Now producing "+destinationLanguage +" code.");
	}

	public void run(Component widget, ActionEvent event) {
		if (outputFileName == null)
            return;
        final File outputFile = new File(outputFileName).getAbsoluteFile();
        final File outputDir = outputFile.getParentFile();
        
        final PrintStream errStream = new PrintStream(
                new ByteArrayOutputStream() {
            @Override
            public synchronized void write(int b) {
                write(new byte[] {(byte) b}, 0, 1);
            }
            @Override
            public synchronized void write(byte[] b, int off, int len) {
                super.write(b, off, len);
                lt.errorText.append(toString());
                reset();
            }            
        });
        try {
            final Shell shell = new Shell();
            shell.setStdErr(errStream);
            shell.chdir(outputDir);
            if (shell.run("javac", outputFile.getName()) != 0)
                return;
            shell.fork("java", 
                    outputFile.getName().replaceAll("\\.java$", ""));                
        } catch (InterruptedException e) {
        } catch (IOException e) {
            lt.errorText.append(e.getLocalizedMessage());
        }
	}

	public void documentation(Component widget, TreeSelectionEvent event) {
		String divider = "---------------------------------" +
				"------------------\n";
		String doc = (String)((DefaultMutableTreeNode)((JTree)widget).getLastSelectedPathComponent()).getUserObject();
		elvisDocItem elvisDoc = docList.get(doc);
		if (elvisDoc == null)
			return;
		StringBuilder content = new StringBuilder();
		content.append(elvisDoc.name+"\n");
		content.append(divider);
		content.append(elvisDoc.description+"\n");
		TreeNode[] path = ((DefaultMutableTreeNode)((JTree)widget).getLastSelectedPathComponent()).getPath();
		String tab = "  ";
		for (int i=path.length-2;i>0;i--) {
			content.append(tab+"Derives From "+
					((DefaultMutableTreeNode)path[i]).getUserObject()+"\n");
			tab+="  ";
		}
		content.append(divider);
		content.append("\nAttributes:\n");
		Enumeration e = elvisDoc.attrs.elements();
		while (e.hasMoreElements())
			content.append(((elvisDocAttribute)e.nextElement()).displayAttr("  ")+"\n\n");
		lt.docViewer.setText(content.toString());
	}

	/*
	 *  LTErrorHandler is a simple error handler that counts the errors it has seen
	 *  and appends the associated error message to the errorText field on the learning
	 *  tool.
	 */
	public class LTErrorHandler implements ErrorHandler {
		boolean anySeen;
		LearningToolGUI lt;
		int counter;
		
		public LTErrorHandler(LearningToolGUI lt){
			this.lt = lt;
			anySeen = false;
			counter = 0;
		}


	
		public boolean handleError(GUIFileException error) {
			counter++;
			lt.errorText.append("\n"+counter+". "+error.getMessage());
			return true;
		}

		public boolean anyErrors() {
			return counter > 0;
		}
		
		public int totalErrors(){
			return counter;
		}
	}
}
