package lib.java;


import java.util.*;

import lib.java.Button;
import errors.BadAttributeException;
import types.GUIAttributes;
import types.Widget;
import types.WidgetDef;
import types.WidgetDef.Derived;
import util.Builders;

// @EName implements Java's JCheckBox.
@Derived(Button.class)
public class CheckButton extends WidgetDef {
	public CheckButton(Widget w, WidgetDef parent) {
		super(w, parent);
	}
	
	public String getLanguage() {
		return "Java";
	}
	
	@Override
	public List<String> getVariables() {
		 return Collections.singletonList(
	                String.format("public JCheckBox %s;\n", id));
	}
	
	public String preamble(){
		return id.concat(" = new JCheckBox();\n");
	}
	
	public String postamble(){
		return "";
	}
	
	public String addAttributes(GUIAttributes attributes) throws 
	BadAttributeException{
		final StringBuilder builder = new StringBuilder();
		    // @EName borderOutlined Changes whether the button's border appears to be flat.
			builder.append(String.format("%s.setBorderPaintedFlat(%s);\n", id,
					attributes.getBoolean("borderOutlined")));
		return builder.toString();
	}
    
    /*
     * Note that this is not the entire list of defaults; rather, some are
     * inherited from Button (as specified by the @Derived(Button.class)
     * annotation on the CheckButton class). Defaults specified here override
     * those of the same name in the base widget type, just as with normal Java
     * inheritance.
     */
    
    private static final Map<String, List<String>> defaults =
         Builders.buildMultiMap(new TreeMap<String, List<String>>())
         .put("borderOutlined", "false") // @EDefaultValue borderOutlined false
         .put("borderPainted", "false")  // @EDefaultValue borderPainted false
         .finish();
	
	public Map<String, List<String>> getDefaults() {
		return defaults;
	}
	
    private static final ArrayList<String> modifiers = Builders.makeCollection(
            new ArrayList<String>(),
            "public boolean getCheckBoxBorderOutline(JCheckBox checkbutton) {\n" +
            "\treturn checkbutton.isBorderPaintedFlat();\n" +
            "}\n" +
            
            "public void setCheckBoxBorderOutline(JCheckBox checkbutton, boolean newOutline) {\n" +
            "checkbutton.setBorderPaintedFlat(newOutline);\n" +
            "}\n");    
    
	public ArrayList<String> getModifiers() {
		return modifiers;
	}
    
    private static final Set<String> imports = Builders.makeCollection(
            new TreeSet<String>(), 
            "import javax.swing.JCheckBox;\n");

	public Set<String> getImports() {
		return imports;
	}
}