package lib.java;

import java.util.*;

import output.java.CodeBuilder;
import output.java.State;

import errors.BadAttributeException;
import types.GUIAttributes;
import types.Widget;
import types.WidgetDef;
import types.WidgetDef.Abstract;
import util.Builders;

/**
 * @EName Clickable is an abstract base for any widget with an onClick attribute
 *        implemented by an addActionListener() method.
 *
 */
@Abstract
public class Clickable extends WidgetDef {

    public Clickable(Widget widget, WidgetDef parent) {
        super(widget, parent);
    }

    @Override
    public String getLanguage() {
        return "Java";
    }

    @Override
    public String preamble() {
        return "";
    }

    private static final Map<String, List<String>> defaults =
        Builders.buildMultiMap(new TreeMap<String, List<String>>())
                .put("onClick", (String) null) // @EDefaultValue onClick null
                .finish();
    
    @Override
    public Map<String, List<String>> getDefaults() {
        return defaults;
    }

    @Override
    public String addAttributes(GUIAttributes attrs)
            throws BadAttributeException {
    	// @EName onClick Sets the handler for the onClick event.
        if (attrs.hasValue("onClick")) {
            String onClick = attrs.getString("onClick");
            
            final CodeBuilder builder = new CodeBuilder(State.METHOD);
            final String handlerClass =
                String.format("temp$%s$OnClickHandler", id); 
            
            builder.beginBlock("")
                .beginClass("class %s implements ActionListener", handlerClass)
                .beginMethod("public void actionPerformed(ActionEvent event)")
                .append("if (controller != null) " + 
                            "controller.%s(%s, event);", onClick, id)
                .finishMethod()
                .finishClass()
                .append("%s.addActionListener(new %s());", id, handlerClass)
                .finishBlock();
            
            return builder.toString();
        }
        return "";
    }

    @Override
    public List<String> getModifiers() {
        return Collections.emptyList();
    }

    private static final Set<String> imports =
        Builders.makeCollection(new TreeSet<String>(),
                "import java.awt.event.*;\n"
            );
    
    @Override
    public Set<String> getImports() {
        return imports;
    }

    @Override
    public List<String> getVariables() {
        return Collections.emptyList();
    }
}
