package lib.java;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;

import errors.BadAttributeException;
import types.GUIAttributes;
import types.Widget;
import types.WidgetDef;
import types.WidgetDef.Derived;
import util.Builders;

/**
 * @EName The combo box for a Java JComboBox.
 * 
 * @author Ritza
 *
 */

@Derived(Clickable.class)
public class ComboBox extends WidgetDef {

	public ComboBox(Widget widget, WidgetDef parent) {
		super(widget, parent);
	}

	@Override
	public String getLanguage() {
		return "Java";
	}
	
	@Override
	public List<String> getVariables() {
		 return Collections.singletonList(
	                String.format("public JComboBox %s;\n", id));
	}

	@Override
	public String preamble() {
		return id.concat(" = new JComboBox();\n");
	}

	/*
	 *  (non-Javadoc)
	 * @see types.WidgetDef#getDefaults()
	 * 
	 * This is where the defaults are defined. The idea is to have a
     * private static field that gets initialized at class loading time with
     * all the code; this way it doesn't get rebuilt each time. Then all that
     * getDefaults() has to do is return this field. 
     * 
     * TODO: what should we set height and width defaults to?
	 */
    
    private static final Map<String, List<String>> defaults =    
        Builders.buildMultiMap(new TreeMap<String, List<String>>())
			.put("list",(String) null) // @EDefaultValue list null
            .finish();
   
    public Map<String,List<String>> getDefaults() {
		return defaults;
	}

	@Override
	public String addAttributes(GUIAttributes attributes) throws BadAttributeException {
		final StringBuilder builder = new StringBuilder();
		// @EName list Contains a list of what items will appear in the ComboBox.
        if (attributes.getAll("list") != null) {
        	List<String> list = attributes.getAll("list");
        	for (int i = 0; i < list.size(); i++)
        		builder.append(String.format("%s.addItem((String) \"%s\");\n",id,list.get(i)));
        }
		return builder.toString();
	}

	@Override
	// TODO: write appropriate modifier methods if needed.
	public List<String> getModifiers() {
		return Collections.emptyList();
	}
	
	/* This is where the import code is defined. The idea is to have a
     * private static field that gets initialized at class loading time with
     * all the code; this way it doesn't get rebuilt each time. Then all that
     * getImports() has to do is return this field. */
	 private static final Set<String> imports = Builders.makeCollection(
	            new TreeSet<String>(), 
	            "import javax.swing.JComboBox;\n"
	    );
	
	public Set<String> getImports() {
		return imports;
	}

}
