package lib.java;

import java.util.*;

import errors.BadAttributeException;
import types.GUIAttributes;
import types.Widget;
import types.WidgetDef;
import util.Builders;
import util.Functional;

/**
 * @EName Implements the JOptionPane style Dialogs in Swing.
 */
public class DialogBox extends WidgetDef {

	public DialogBox(Widget widget, WidgetDef parent){
		super(widget,parent);
	}

	@Override
	public String getLanguage() {
		return "";
	}

	@Override
	public String preamble() {
		return "";
	}

	
    private static final Map<String, List<String>> defaults =
        Builders.buildMultiMap(new TreeMap<String, List<String>>())
                .put("aspect", "message") 	//@EDefaultValue aspect message
                .put("message", "Message!")	//@EDefaultValue message Message!
                .put("title", "Message!")	//@EDefaultValue title Message!
                .put("buttons", "_DEFAULT")	//@EDefaultValue buttons "_DEFAULT"
                .finish();
	@Override
	public Map<String, List<String>> getDefaults() {
		return defaults;
	}

    private Aspect aspect;
    private String[] message;
    private String[] title;
    private Buttons buttons;
    
    private boolean aspectSet = false;
    private boolean buttonsSet = false;
    
    private enum Aspect {
        message, question, info, input, error, warning
    }
    
    private enum Buttons {
        _DEFAULT, yes, yes_no, yes_no_cancel, ok_cancel
    }
    
	@Override
	public String addAttributes(GUIAttributes attrs)
			throws BadAttributeException {
		//@EName aspect Specifies the aspect of the dialog box: question, info, input, warning, error or message.
        aspect = attrs.getEnum("aspect", Aspect.class);
        //@EName message Specifies the text content of the dialog box. Can be reset at runtime.
        message = attrs.getStrings("message").toArray(new String[0]);
        //@EName title Specifies the title of the dialog box.
        title = attrs.getStrings("title").toArray(new String[0]);
        //@EName buttons Specifies the buttons to allow on the dialog: _DEFAULT, yes, yes_no, yes_no_cancel or ok_cancel.
        buttons = attrs.getEnum("buttons", Buttons.class);
        
        aspectSet = 
            attrs.getStrings("aspect").equals(defaults.get("aspect"));
        buttonsSet = 
            attrs.getStrings("buttons").equals(defaults.get("buttons"));
        return "";
	}

	@Override
	public List<String> getModifiers() {
		String returnType = "void";
		String messageType = "showMessageDialog";
		boolean optionDialog = false;
		boolean inputDialog = false;
		String optionType = "JOptionPane.DEFAULT_OPTION";
		String iconType = "JOptionPane.PLAIN_MESSAGE";
		
		if (aspectSet) {
            switch (aspect) {
                case question:
                    iconType = "JOptionPane.QUESTION_MESSAGE";
                    messageType = "showConfirmDialog";
                    returnType = "int";
                    optionDialog = true;
                    
                    if (buttonsSet)
                        switch (buttons) {
                            case yes:
                                optionType = "JOptionPane.YES_OPTION";
                                break;
                            case yes_no: 
                                optionType = "JOptionPane.YES_NO_OPTION";
                                break;
                            case yes_no_cancel: 
                                optionType = "JOptionPane.YES_NO_CANCEL_OPTION";
                                break;
                            case ok_cancel:
                                optionType = "JOptionPane.OK_CANCEL_OPTION";
                                break;
                        }
                    break;
			
                case input:
                    iconType = "JOptionPane.QUESTION_MESSAGE";
                    messageType = "showInputDialog";
                    returnType = "String";
                    inputDialog = true;
                    break;
                    
                case info:
                    iconType = "JOptionPane.INFORMATION_MESSAGE";
                    break;
			
                case error:
                    iconType = "JOptionPane.ERROR_MESSAGE";
                    break;
			
                case warning:
                    iconType = "JOptionPane.WARNING_MESSAGE";
            }
		}
		

        String message = "\""+Functional.join(this.message, " ")+"\"";
		
        String title = "\""+Functional.join(this.title," ")+"\"";
		
		if(optionDialog)
			return Builders.makeCollection(new ArrayList<String>(),String.format("public %s trigger%sDialog(){\n\t return JOptionPane.%s(null,%s,%s,%s,%s);\n}",
						returnType,widget.id,messageType,message,title,optionType,iconType),
						String.format("public %s trigger%sDialog(String message){\n\t return JOptionPane.%s(null,message,%s,%s,%s);\n}",
								returnType,widget.id,messageType,title,optionType,iconType));
		else if (inputDialog)
			return Builders.makeCollection(new ArrayList<String>(),String.format("public %s trigger%sDialog(){\n\t return JOptionPane.%s(null,%s,%s,%s);\n}",
						returnType,widget.id,messageType,message,title,iconType),
						String.format("public %s trigger%sDialog(String message){\n\t return JOptionPane.%s(null,message,%s,%s);\n}",
								returnType,widget.id,messageType,title,iconType));
		else
			return Builders.makeCollection(new ArrayList<String>(),String.format("public %s trigger%sDialog(){\n\tJOptionPane.%s(null,%s,%s,%s);\n}",
						returnType,widget.id,messageType,message,title,iconType),
						String.format("public %s trigger%sDialog(String message){\n\tJOptionPane.%s(null,message,%s,%s);\n}",
								returnType,widget.id,messageType,title,iconType));
	}

	@SuppressWarnings("unchecked")
	@Override
	public Set<String> getImports() {
		return Builders.makeCollection(new TreeSet(),"javax.swing.JOptionPane");
	}

    @Override
    public List<String> getVariables() {
        return Collections.emptyList();
    }

}
