package lib.java;

import java.util.*;

import types.GUIAttributes;
import types.Widget;
import types.WidgetDef;
import types.WidgetDef.*;

/**
 * Abstract widget type for any Layout.
 * <p>
 * This type actually does nothing; it's much like a tagging interface in Java
 * - that is, one that has no members, and which is only used in testing
 * membership in a class. See the code for the Window widget type.
 * 
 * @EName Abstract Widget governing all layouts.
 */

@Abstract @Contained
public class Layout extends WidgetDef {
    public Layout(Widget widget, WidgetDef parent) {
        super(widget, parent);
    }
    
    public String getLanguage() {
        return "Java";
    }
    
    public String preamble() {
        return "";
    }
    
    public String addAttributes(GUIAttributes attrs) {
        return "";
    }
    
    public Map<String, List<String>> getDefaults() {
        return Collections.emptyMap();
    }

    @Override
    public String addChild(String childType, String childId,
            GUIAttributes attrs) {
        return String.format("\n// Add %s to %s\n",
                commentForm(childId, childType),
                getCommentName());
    }
    
    public Set<String> getImports() {
        return Collections.emptySet();
    }

    public List<String> getModifiers() {
        return Collections.emptyList();
    }

    public List<String> getVariables() {
        return Collections.emptyList();
    }
}
