package lib.java;

import java.util.*;

import errors.BadAttributeException;
import types.GUIAttributes;
import types.Widget;
import types.WidgetDef;
import util.Builders;

/**
 * @EName Menu Separator for a JMenu.  Does not have any attributes.
 *
 */
public class MenuSeparator extends WidgetDef{

	public MenuSeparator(Widget widget, WidgetDef parent) {
        super(widget, parent);
    }

	@Override
	public String getLanguage() {
		return "Java";
	}

	@Override
	public String preamble() {
		return id + " = new JSeparator();\n";
	}

	@Override
	public String postamble() {
		return "";
	}

	@Override
	public String addAttributes(GUIAttributes attributes) throws BadAttributeException {
		return "";
	}

	@Override
	public Map<String, List<String>> getDefaults() {
		return new TreeMap<String, List<String>>();
	}

	@Override
	public List<String> getModifiers() {
		return new ArrayList<String>();
	}

	/* This is where the import code is defined. The idea is to have a
     * private static field that gets initialized at class loading time with
     * all the code; this way it doesn't get rebuilt each time. Then all that
     * getImports() has to do is return this field. */
	 private static final Set<String> imports = Builders.makeCollection(
	            new TreeSet<String>(), 
	            "import javax.swing.JSeparator;\n");

	@Override
	public Set<String> getImports() {
		return imports;
	}

    @Override
    public List<String> getVariables() {
        return Collections.singletonList("public JSeparator " + id + ";\n");
    }

}
