package lib.java;

import java.util.*;

import errors.BadAttributeException;
import errors.BadChildWidgetException;
import types.GUIAttributes;
import types.Widget;
import types.WidgetDef;
import util.Builders;

/**
 * RadioButtonGroup definition for a Java ButtonGroup.  This definition contains
 * no attributes; only the preamble and the ability to add RadioButtons as 
 * children.  
 * @Ename RadioButtonGroup A widget to unite a family of radio buttons, with only radio buttons as legitimate children.
 * @author ritza
 * @see types.WidgetDef for more information about the methods.
 *
 */
public class RadioButtonGroup extends WidgetDef{

	public RadioButtonGroup(Widget widget, WidgetDef parent) {
        super(widget, parent);
    }

	public String getLanguage() {
		return "Java";
	}

	public String preamble() {
		return id + " = new ButtonGroup();\n";
	}

	public String postamble() {
		return "";
	}

	public String addAttributes(GUIAttributes attributes) throws BadAttributeException {
		return "";
	}


	public Map<String, List<String>> getDefaults() {
		return Collections.emptyMap();
	}

    @Override
	public String addChild(String childType, String childName, 
            GUIAttributes attrs) throws BadChildWidgetException {
		childType = childType.toLowerCase();
		if (childType.equals("radiobutton"))
			return id.concat(".add(").concat(childName).concat(");\n");
		throw new BadChildWidgetException(widget);    
	}


	public List<String> getModifiers() {
		return Collections.emptyList();
	}

	/* This is where the import code is defined. The idea is to have a
	 * private static field that gets initialized at class loading time with
	 * all the code; this way it doesn't get rebuilt each time. Then all that
	 * getImports() has to do is return this field. */
	private static final Set<String> imports;
	static {
		imports = Builders.makeCollection(
				new TreeSet<String>(), 
				"import javax.swing.ButtonGroup;\n");
	}
	
	public Set<String> getImports() {
		return imports;
	}

    @Override
    public List<String> getVariables() {
        return Collections.singletonList("public ButtonGroup " + id + ";\n");
    }
}
