package lib.java;

import java.util.*;

import errors.BadAttributeException;
import errors.BadChildWidgetException;
import types.GUIAttributes;
import types.Widget;
import types.WidgetDef;
import util.Builders;

/**
 * @EName Nodes that a Tree contains.
 *
 */
public class TreeNode extends WidgetDef {

	public TreeNode(Widget widget, WidgetDef parent) {
        super(widget, parent);
    }
    
	@Override
	public String getLanguage() {
		return "Java";
	}

	@Override
	public String preamble() {
		return id + " = new DefaultMutableTreeNode();\n";
	}
	
	public String postamble() {
		return "";
	}

	@Override
	public String addAttributes(GUIAttributes attributes) throws BadAttributeException {
		final StringBuilder builder = new StringBuilder();
		//@EName text text for the TreeNode
		if (attributes.getOne("text") != null)
		    builder.append(String.format("%s.setUserObject(\"%s\");\n",id,
		            attributes.getOne("text")));
		return builder.toString();
	}

	private static final Map<String, List<String>> defaults =
        Builders.buildMultiMap(new TreeMap<String, List<String>>())
            .put("text", (String) null) //@EDefaultValue text null
            .finish();
	
	@Override
	public Map<String, List<String>> getDefaults() {
		return defaults;
	}
	
	private static final Map<String, String> synonyms =
		Builders.buildMap(new TreeMap<String, String>())
		.put("label", "text")
		.finish();
	
	@Override
	public Map<String, String> getSynonymousAttributes() {
		return synonyms;
	}
	
    @Override
	public String addChild(String childType, String childName,
            GUIAttributes childAttrs)
    throws BadChildWidgetException {
		if (childType.equals("TreeNode"))
			return String.format("%s.add(%s);\n",id,childName);
		else
			throw new BadChildWidgetException(widget);
}
	
	/*
	 * This is where the defaults are defined. The idea is to have a
     * private static field that gets initialized at class loading time with
     * all the code; this way it doesn't get rebuilt each time. Then all that
     * getDefaults() has to do is return this field. 
	 */
	
	private static final ArrayList<String> modifiers = Builders.makeCollection(
			new ArrayList<String>(),
			"public String getTreeNodeLabel(DefaultMutableTreeNode treeNode) {\n" +
			"\treturn (String) treeNode.getUserObject();\n" +
			"}\n" +
			
			"public void setTreeNodeLabel(DefaultMutableTreeNode treeNode, String newLabel) {\n" +
			"\ttreeNode.setUserObject(newLabel);\n" +
			"}\n"
		);
	
	@Override
	public List<String> getModifiers() {
		return modifiers;
	}

	/* This is where the import code is defined. The idea is to have a
     * private static field that gets initialized at class loading time with
     * all the code; this way it doesn't get rebuilt each time. Then all that
     * getImports() has to do is return this field. */
	 private static final Set<String> imports = Builders.makeCollection(
	            new TreeSet<String>(), 
                "import javax.swing.tree.DefaultMutableTreeNode;\n");

	@Override
	public Set<String> getImports() {
		return imports;
	}

    @Override
    public List<String> getVariables() {
        return Collections.singletonList(
                "public DefaultMutableTreeNode " + id + ";\n");
    }

}