package lib.java;

import java.util.*;

import errors.BadAttributeException;

import output.java.CodeBuilder;
import output.java.State;

import types.GUIAttributes;
import types.WidgetDef;
import types.WidgetDef.Abstract;
import util.Builders;
/*
 * @EName Widget Abstract Class that serves as a root for all other
 * widgets.
 * 
 *
 */
@Abstract
public class Widget extends WidgetDef {

    public Widget(types.Widget widget, WidgetDef parent) {
        super(widget, parent);
    }

    @Override
    public String getLanguage() {
        return "Java";
    }

    @Override
    public String preamble() {
        return new CodeBuilder(State.METHOD)
            .append("\n")
            .appendComment("Construct %s %s", widget.type,
                    id.startsWith("anon$") ? "(anonymous)" : id)
            .finish();
    }
    
    @Override
    public List<String> getVariables() {
        return Collections.emptyList();
    }

    private static final Map<String, List<String>> defaults =
        Builders.buildMultiMap(new TreeMap<String, List<String>>())
            // .put("width", "-1")
            // .put("height", "-1")
            .put("minWidth", "-1")
            .put("minHeight", "-1")
            .put("preferredWidth", "-1")
            .put("preferredHeight", "-1")
            .put("maxWidth", "-1")
            .put("maxHeight", "-1")
            .finish();
    
    @Override
    public Map<String, List<String>> getDefaults() {
        return defaults;
    }

    private static final Map<String, List<String>> multisynonyms =
        Builders.buildMultiMap(new TreeMap<String, List<String>>())
            // .put("size", "width", "height")
            .put("minSize", "minWidth", "minHeight")
            .put("preferredSize", "preferredWidth", "preferredHeight")
            .put("maxSize", "maxWidth", "maxHeight")
            .finish();
    
    @Override
    public Map<String, List<String>> getMultisynonymousAttributes() {
        return multisynonyms;
    }
    
    @Override
    public String addAttributes(GUIAttributes attrs) 
            throws BadAttributeException {
        CodeBuilder builder = new CodeBuilder();
        int minWidth = attrs.getInteger("minWidth"),
            minHeight = attrs.getInteger("minHeight"),
            preferredWidth = attrs.getInteger("preferredWidth"),
            preferredHeight = attrs.getInteger("preferredHeight"),
            maxWidth = attrs.getInteger("maxWidth"),
            maxHeight = attrs.getInteger("maxHeight");
        boolean needDimensionImport = false;
        
        if (minWidth != -1 || minHeight != -1) {
            if (minWidth == -1)
                minWidth = 0;
            else if (minHeight == -1)
                minHeight = 0;
            builder.append("%s.setMinimumSize(new Dimension(%d, %d));",
                    id, minWidth, minHeight);
            needDimensionImport = true;
        }
        
        if (preferredWidth != -1 && preferredHeight != -1) {
            builder.append("%s.setPreferredSize(new Dimension(%d, %d));",
                    id, minWidth, minHeight);
            needDimensionImport = true;
        }
        
        if (maxWidth != -1 || maxHeight != -1) {
            if (maxWidth == -1)
                maxWidth = Integer.MAX_VALUE;
            else if (maxHeight == -1)
                maxHeight = Integer.MAX_VALUE;
            builder.append("%s.setMinimumSize(new Dimension(%d, %d));",
                    id, maxWidth, maxHeight);
            needDimensionImport = true;
        }
        
        if (needDimensionImport)
            imports.add("java.awt.Dimension");
        
        return builder.finish();
    }

    @Override
    public List<String> getModifiers() {
        return Collections.emptyList();
    }

    private Set<String> imports = new TreeSet<String>();
    
    @Override
    public Set<String> getImports() {
        return imports;
    }

}
