#!/usr/bin/perl

use strict;
use Data::Dumper;

my $source = "ParserTreeConstants.java";
my $output = $source;
# my $output = "/dev/stdout";

open INFILE, '<', $source;

my $marker = "/* EDITED BY mkConstants.pl */\n";
my @lines = ($marker);

while (<INFILE>) {
    exit if $_ eq $marker;
    push @lines, $_;
    last if (/^{/);
}

push @lines, "  enum Kind {\n";

# Rewrite the constants as enumeration items

my @consts = ();

while (<INFILE>) {    
    /public int JJT(\w+) = (\d+);/ or last;
    push @consts, [$1, $2];
}

my @names = ();

while (<INFILE>) {
    next if /^\s$/; # Skip further blank lines
    next if /public String\[\] jjtNodeName = {/; # Skip array declaration
    /(\".*\"),/ or last;
    push @names, $1;
}

for (my $ix = 0; $ix <= $#consts; $ix++) {
    my ($id, $value) = @{$consts[$ix]};
    my $name = $names[$ix];
    push @lines, "    $id($value, $name),\n";
}

push @lines, <<EOF;
    ;

    public final int value;
    public final String nontermId;
    
    Kind(int value, String nontermId) {
      this.value = value;
      this.nontermId = nontermId;
    }

    public Kind kindWithNontermId(String id) {
      for (Kind kind : values())
        if (kind.nontermId.equals(id))
          return kind;
      return null;
    }

    public String toString() {
      return nontermId;
    }
  }

EOF

for my $const (@consts) {
    my ($id, $value) = @$const;
    push @lines, "  int JJT$id = Kind.$id.value;\n"
}

push @lines, "}\n";

close INFILE;
open OUTFILE, '>', $output;
print OUTFILE @lines;
